SystemOrganization addCategory: #'AL-Application-Core'!
SystemOrganization addCategory: #'AL-Application-Utilities'!

!SmallInteger methodsFor: '*al-application-printing' stamp: 'cwp 7/10/2012 11:47'!
numberOfDigitsInBase66
	| nDigits q total |
	self negative ifTrue: [ ^ self negated numberOfDigitsInBase66 ].
	self < 66 ifTrue: [ ^ 1 ].
	q := self.
	total := 0.
	[nDigits := q highBit quo: 7.
	total := total + nDigits.
	(q := q quo: (66 raisedToInteger: nDigits)) < 66 ] whileFalse.
	^ q = 0
		ifTrue: [ total ]
		ifFalse: [ total + 1 ]! !

!SmallInteger methodsFor: '*al-application-printing' stamp: 'cwp 7/10/2012 11:41'!
streamingPrintBase66On: aWriteStream
	| n divisor |
	divisor := 66 raisedToInteger: (super numberOfDigitsInBase: 66) - 1.
	n := self.
	[divisor > 0] whileTrue: 
		[| digit |
		digit := n // divisor.
		aWriteStream put: ('0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz-_.~' at: digit + 1).
		n := n - (digit * divisor).
		divisor := divisor // 66]! !

!SmallInteger methodsFor: '*al-application-printing' stamp: 'cwp 7/10/2012 11:43'!
streamingPrintBase66On: aWriteStream digits: anInteger
	| totalLength |
	totalLength := self numberOfDigitsInBase66.
	[totalLength < anInteger] whileTrue: 
		[aWriteStream put: $0.
		totalLength := totalLength + 1].
	self streamingPrintBase66On: aWriteStream! !

Object subclass: #ALApplication
	instanceVariableNames: 'handler locator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALApplication class methodsFor: 'as yet unclassified' stamp: 'cwp 3/4/2012 12:01'!
new
	^ self withLocator: (ALDigestLocator new)! !

!ALApplication class methodsFor: 'as yet unclassified' stamp: 'cwp 3/7/2011 23:08'!
withLocator: aLocator
	^ self basicNew
		initializeWithLocator: aLocator;
		yourself! !

!ALApplication methodsFor: 'api' stamp: 'cwp 9/12/2011 22:50'!
addRelay: aRelay
	aRelay destination: handler.
	handler := aRelay! !

!ALApplication methodsFor: 'error handling' stamp: 'cwp 3/31/2012 20:16'!
handleMethodNotAllowed: anException
	^ ALResponse
		status: 405
		reason: 'Method not allowed'
	! !

!ALApplication methodsFor: 'error handling' stamp: 'cwp 3/14/2012 13:29'!
handleNotAcceptable: anError
	^ ALResponse status: 406 reason: 'Not Acceptable'! !

!ALApplication methodsFor: 'error handling' stamp: 'cwp 4/23/2011 18:44'!
handleNotFound: anError
	^ ALResponse notFound: anError messageText! !

!ALApplication methodsFor: 'api' stamp: 'cwp 10/15/2011 00:01'!
handleRequest: aRequest
	^ [ handler handleRequest: aRequest ] 
		on: ALHttpError 
		do:	[:err | 
			aRequest consume.
			err handleWith: self]! !

!ALApplication methodsFor: 'error handling' stamp: 'cwp 4/1/2012 19:32'!
handleUnprocessableEntity: anError
	^ ALRequest
		status: 422
		reason: 'Unprocessable entity'
		details: anError messageText! !

!ALApplication methodsFor: 'error handling' stamp: 'cwp 10/14/2011 23:53'!
handleUnsupportedContentType: anError 
	^ ALResponse 
		status: 415 
		reason: 'Unsupported Content Type'
		details: anError messageText ! !

!ALApplication methodsFor: 'initialize-release' stamp: 'cwp 8/2/2012 22:54'!
initializeHandler
	handler := ALChunkingRelay destination: handler.
	handler := ALKeepAliveRelay destination: handler.
	handler := ALRequestDecodingRelay destination: handler.
	handler := ALLimitingRelay destination: handler! !

!ALApplication methodsFor: 'initialize-release' stamp: 'cwp 9/27/2011 16:20'!
initializeLocator! !

!ALApplication methodsFor: 'initialize-release' stamp: 'cwp 9/27/2011 16:24'!
initializeWithLocator: aLocator
	locator := aLocator.
	handler := aLocator.
	self initializeLocator.
	self initializeHandler.! !

!ALApplication methodsFor: 'api' stamp: 'cwp 7/13/2012 22:40'!
write: aMessage into: aStream
	aMessage writeInto: aStream! !

Object subclass: #ALDirectoryScanner
	instanceVariableNames: 'locator guide ignore root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Utilities'!

!ALDirectoryScanner class methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:34'!
locator: aLocator
	^ self basicNew initializeWithLocator: aLocator  ! !

!ALDirectoryScanner class methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:46'!
scan: aReference into: aLocator
	(self locator: aLocator) scan: aReference.
	^ aLocator! !

!ALDirectoryScanner methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:37'!
initializeWithLocator: aLocator
	locator := aLocator.
	guide := FSPreorderGuide for: self.! !

!ALDirectoryScanner methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:54'!
scan: aReference
	root := aReference resolve.
	guide show: root! !

!ALDirectoryScanner methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:53'!
visitDirectory: anEntry
	| reference |
	reference := anEntry reference.
	(reference basename first = $.) ifTrue: [ignore := reference].
	! !

!ALDirectoryScanner methodsFor: 'as yet unclassified' stamp: 'cwp 8/17/2011 00:00'!
visitFile: anEntry
	| path reference |
	reference := anEntry reference.
	ignore ifNotNil: [(ignore contains: reference) ifTrue: [^ self]].
	ignore := nil.
	(reference basename first = $.) ifFalse:
		[path := ALPath withAll: (reference relativeTo: root).
		locator at: path put: (ALFileResource on: reference)]! !

Object subclass: #ALLocator
	instanceVariableNames: 'paths resources'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Core'!

ALLocator subclass: #ALDigestLocator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALDigestLocator methodsFor: 'private' stamp: 'cwp 3/10/2012 22:11'!
keyForResource: aResource
	^ self printDigest: aResource serialize digest! !

!ALLocator methodsFor: 'public' stamp: 'cwp 3/4/2012 19:32'!
at: aPath alias: aResource
	resources at: aPath put: aResource.! !

!ALLocator methodsFor: 'public' stamp: 'cwp 3/4/2012 19:39'!
at: aPath put: aResource
	resources at: aPath put: aResource.
	paths 
		at: (self keyForResource: aResource) 
		put: aPath! !

!ALLocator methodsFor: 'public' stamp: 'cwp 3/4/2012 11:46'!
handleRequest: aRequest
	| path |
	path := aRequest path.
	^ (self resourceForPath: path) handleRequest: aRequest locator: self! !

!ALLocator methodsFor: 'initialize-release' stamp: 'cwp 3/5/2012 21:02'!
initialize
	super initialize.
	paths := Dictionary new.
	resources := Dictionary new.! !

!ALLocator methodsFor: 'private' stamp: 'cwp 3/5/2012 21:02'!
keyForResource: aResource
	self subclassResponsibility! !

!ALLocator methodsFor: 'public' stamp: 'cwp 3/4/2012 19:33'!
pathForResource: aResource
	| key |
	key := self keyForResource: aResource.
	^ paths
		at: key
		ifAbsentPut: [ 
			| path |
			path := ALPath with: key.
			resources at: path put: aResource.
			path ]! !

!ALLocator methodsFor: 'private' stamp: 'cwp 7/10/2012 11:41'!
printDigest: arg
	"This is basically the LargeInteger implementation, with constant propagation applied"

	| out |
	arg primitiveChangeClassTo: 1073741824.
	out := String new writing.
	arg streamingPrintBase66On: out.
	^ out conclusion.! !

!ALLocator methodsFor: 'public' stamp: 'cwp 3/5/2012 20:55'!
resourceForPath: aPath
	^ resources at: aPath ifAbsent: 
		[ALNotFound signal: aPath printString]
		
			! !

!ALLocator methodsFor: 'serializing' stamp: 'cwp 3/6/2012 23:37'!
serializeWith: aSerializer
	aSerializer serializeExternally: self  ! !

ALLocator subclass: #ALTrivialLocator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALTrivialLocator methodsFor: 'private' stamp: 'cwp 3/4/2012 19:35'!
keyForResource: aResource
	^ resources size asString! !

Object variableSubclass: #ALPath
	instanceVariableNames: ''
	classVariableNames: 'Slash'
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/20/2011 22:57'!
/ aString
	^ self with: aString! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 9/6/2011 16:16'!
initialize
	Slash := '/'! !

!ALPath class methodsFor: 'dispatch' stamp: 'cwp 10/25/2011 13:45'!
parseWith: aParser
	^ aParser parsePath! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 14:31'!
root
	^ self new: 0! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 14:48'!
with: aString
	^ (self new: 1)
		at: 1 put: aString;
		yourself! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:58'!
withAll: segments
	| path |
	path := self new: segments size.
	1 to: path size do: [:index | path at: index put: (segments at: index)].
	^ path! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 9/13/2011 10:49'!
/ aString
	| path |
	(self isRoot and: [aString isEmpty]) ifTrue: [^ self].
	path := self class new: self size + 1.
	1 to: self size do: [:i | path at: i put: (self at: i)].
	path at: (path size) put: aString.
	^ path! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/21/2011 22:20'!
= other
	self species = other species ifFalse: [^ false].
	self size = other size ifFalse: [^ false].
	1 to: self size do: [:i | (self at: i) = (other at: i) ifFalse: [^ false]].
	^ true! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 7/11/2012 22:57'!
asString
	^ self printString! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 15:17'!
first
	^ self at: 1! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/21/2011 22:22'!
hash
	| hash |
	hash := self class hash.
	1 to: self size do: [:i | hash := hash bitXor: (self at: i) hash].
	^ hash! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 14:54'!
isRoot
	^ self size = 0! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 8/21/2011 22:16'!
printOn: aStream
	| ws |
	ws := String new writing.
	self streamingPrintOn: ws.  
	aStream nextPutAll: ws conclusion.! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 7/11/2012 22:56'!
printString
	^ String new writing
		print: self;
		conclusion! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 15:18'!
rest
	| rest |
	rest := self class new: self size - 1.
	1 to: rest size do: [:index | rest at: index put: (self at: index + 1)].
	^ rest! !

!ALPath methodsFor: 'as yet unclassified' stamp: 'cwp 9/6/2011 16:15'!
streamingPrintOn: aWriteStream
	self isRoot 
		ifTrue: [aWriteStream write: Slash]
		ifFalse:
			[1 to: self size do:
				[:index |
				aWriteStream write: Slash.
				aWriteStream write: (self at: index)]]! !

Object subclass: #ALResource
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Core'!

ALResource subclass: #ALCallbackResource
	instanceVariableNames: 'resource callback'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALCallbackResource class methodsFor: 'as yet unclassified' stamp: 'cwp 8/5/2011 22:19'!
resource: aResource callback: aValuable
	^ self basicNew initializeWithResource: aResource callback: aValuable! !

!ALCallbackResource methodsFor: 'as yet unclassified' stamp: 'cwp 8/21/2011 22:30'!
applyRequest: aRequest
	^ self value! !

!ALCallbackResource methodsFor: 'public' stamp: 'cwp 10/1/2011 16:28'!
handleGet: aRequest locator: aLocator
	| path |
	path := aLocator pathForResource: (self applyRequest: aRequest).
	^ ALResponse seeOther: path! !

!ALCallbackResource methodsFor: 'as yet unclassified' stamp: 'cwp 8/5/2011 22:20'!
initializeWithResource: aResource callback: aValuable
	self initialize.
	resource := aResource.
	callback := aValuable.! !

!ALCallbackResource methodsFor: 'as yet unclassified' stamp: 'cwp 8/21/2011 22:30'!
value
	callback value.
	^ resource! !

ALResource subclass: #ALFileResource
	instanceVariableNames: 'reference'
	classVariableNames: 'MimeTypes'
	poolDictionaries: ''
	category: 'AL-Application-Core'!

!ALFileResource class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 13:04'!
initialize
	MimeTypes := Dictionary new
		at: 'css' put: ALContentType css;
		at: 'html' put: ALContentType html;
		at: 'ico' put: ALContentType favicon;
		at: 'js' put: ALContentType javascript;
		at: 'less' put: ALContentType less;
		at: 'png' put: ALContentType png;
		yourself.! !

!ALFileResource class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 23:17'!
on: aReference
	^ self new initializeWithReference: aReference! !

!ALFileResource methodsFor: 'comparing' stamp: 'cwp 3/11/2011 23:18'!
= other 
	^ self species = other species and: [self reference = other reference]! !

!ALFileResource methodsFor: 'public' stamp: 'cwp 8/22/2011 22:19'!
contentType
	^ MimeTypes 
		at: reference extension
		ifAbsent: ['application/octet-stream']! !

!ALFileResource methodsFor: 'public' stamp: 'cwp 10/18/2011 16:56'!
handleGet: aRequest locator: aLocator
	| body |
	reference exists ifFalse: [ALNotFound signal].
	body := ALEntity callback: 
		[ :out || in | 
		in := reference reading.
		[out write: reference reading]
			ensure: [in close]].
	body contentType: self contentType.
	body contentLength: reference resolve entry size.
	^ ALResponse status: 200 reason: 'OK' body: body! !

!ALFileResource methodsFor: 'comparing' stamp: 'cwp 3/11/2011 23:18'!
hash
	^ reference hash! !

!ALFileResource methodsFor: 'initialize-release' stamp: 'cwp 3/11/2011 23:18'!
initializeWithReference: aReference
	reference := aReference ! !

!ALFileResource methodsFor: 'accessing' stamp: 'cwp 3/11/2011 23:18'!
reference
	^ reference! !

!ALResource methodsFor: 'converting' stamp: 'cwp 3/5/2012 21:22'!
asResource
	^ self! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleConnect: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleDelete: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleGet: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleHead: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleOptions: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/6/2011 21:22'!
handlePost: aRequest locator: aLocator
	aRequest readEntityWith: [:in | in rest].  
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/6/2011 21:22'!
handlePut: aRequest locator: aLocator
	aRequest readEntityWith: [:in | in rest].  
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'public' stamp: 'cwp 10/1/2011 16:24'!
handleRequest: aRequest locator: aLocator
	aRequest isMethodConnect ifTrue: [^ self handleConnect: aRequest locator: aLocator].
	aRequest isMethodDelete ifTrue: [^ self handleDelete: aRequest locator: aLocator].
	aRequest isMethodGet ifTrue: [^ self handleGet: aRequest locator: aLocator].
	aRequest isMethodHead ifTrue: [^ self handleHead: aLocator locator: aLocator].
	aRequest isMethodOptions ifTrue: [self handleOptions: aRequest locator: aLocator].
	aRequest isMethodPost ifTrue: [^ self handlePost: aRequest locator: aLocator].
	aRequest isMethodPut ifTrue: [^ self handlePut: aRequest locator: aLocator].
	aRequest isMethodTrace ifTrue: [^ self handleTrace: aRequest locator: aLocator].
	self methodNotAllowed: aRequest method.! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:41'!
handleTrace: aRequest locator: aLocator
	^ self methodNotAllowed: aRequest  ! !

!ALResource methodsFor: 'private' stamp: 'cwp 10/1/2011 16:42'!
methodNotAllowed: aRequest
	^ ALResponse
		status: 405
		reason: 'Method not allowed'
		details: 'The method ', aRequest method, ' is not allowed for ', aRequest path asString! !

!LargePositiveInteger methodsFor: '*al-application-printing' stamp: 'cwp 7/10/2012 11:41'!
streamingPrintBase66On: aWriteStream
	| halfDigits halfPower head tail nDigitsUnderestimate |
	(self digitLength = 0 or: [ (self digitAt: self digitLength) = 0 ])
		ifTrue: [ ^ self normalize streamingPrintBase66On: aWriteStream ].
	nDigitsUnderestimate := self highBit quo: 6.
	halfDigits := 1 bitShift: nDigitsUnderestimate highBit - 2.
	halfPower := 66 raisedToInteger: halfDigits.
	head := self quo: halfPower.
	tail := self - (head * halfPower).
	head streamingPrintBase66On: aWriteStream.
	tail streamingPrintBase66On: aWriteStream digits: halfDigits! !

!LargePositiveInteger methodsFor: '*al-application-printing' stamp: 'cwp 7/10/2012 11:41'!
streamingPrintBase66On: aWriteStream digits: halfDigits
	| halfPower half head tail |
	halfPower := halfDigits bitShift: -1.
	half := 66 raisedToInteger: halfPower.
	head := self quo: half.
	tail := self - (head * half).
	head streamingPrintBase66On: aWriteStream digits: halfDigits - halfPower.
	tail streamingPrintBase66On: aWriteStream digits: halfPower ! !
ALPath initialize!
ALFileResource initialize!
